
import requests

class RDPClient:
    def __init__(self, base_url="https://rdp.ucc.ie/api", auth_token=None):
        self.base_url = base_url
        self.auth_token = auth_token

    def make_api_request(self, endpoint, params=None):
        headers = {
            'Authorization': f'Token {self.auth_token}',
            'Accept': 'application/json'
        } if self.auth_token else {
            'Accept': 'application/json'
        }

        response = requests.get(f'{self.base_url}/{endpoint}', params=params, headers=headers)
        response.raise_for_status()  # Raise an exception for HTTP errors
        return response.json()

    def list_sample_fields(self):
        return self.make_api_request('samples/fields/')

    def query_samples(self, query, fields=None, limit=100):
        params = {'query': query, 'fields': fields, 'limit': limit}
        return self.make_api_request('samples', params=params)



from RDPTools.inflate import inflate_fasta, inflate_bam
import gzip
import hashlib
import pysam
import pytest


def test_inflate_bam():
    infile = 'tests/test_data/test.bam'
    outfile = 'tests/test_data/test.inflated.bam'
    inflate_bam(infile, outfile)
    pysam.index(outfile)
    bam = pysam.AlignmentFile(outfile, 'rb')
    assert bam.count() == 158312

def test_inflate_fasta():
    inflate_fasta(
        'tests/test_data/reads.collapsed.fa.gz',
        'tests/test_data/test.inflated.fa')
    with open('tests/test_data/test.inflated.fa', 'rb') as f:
        m = hashlib.md5()
        m.update(f.read())
        assert m.hexdigest() == 'b36b91d8eb1b407324ffa0c6fa50f8e6'

def test_inflate_fasta_gz():
    inflate_fasta(
        'tests/test_data/reads.collapsed.fa.gz',
        'tests/test_data/test.inflated.fa.gz',
        compress=True)
    # these files must be read differently as gzip files from python automatically
    # add the metadata (no -n flag) so MD5 hashes will not match
    with gzip.open('tests/test_data/test.inflated.fa.gz', 'rb') as f:
        content = f.read()
    
    md5_hash = hashlib.md5(content).hexdigest()
    assert md5_hash == 'b36b91d8eb1b407324ffa0c6fa50f8e6'

def test_inflate_fasta_to_fastq():
    infile = 'tests/test_data/reads.collapsed.fa'
    outfile = 'tests/test_data/test.inflated.fq'
    inflate_fasta(infile, outfile)
    with open(outfile, 'rb') as f:
        m = hashlib.md5()
        m.update(f.read())
        assert m.hexdigest() == 'd41d8cd98f00b204e9800998ecf8427e'

def test_inflate_fasta_to_fastq_gz():
    inflate_fasta(
        'tests/test_data/reads.collapsed.fa.gz',
        'tests/test_data/test.inflated.fq.gz',
        compress=True)

    with gzip.open('tests/test_data/test.inflated.fq.gz', 'rb') as f:
        content = f.read()
    
    md5_hash = hashlib.md5(content).hexdigest()
    assert md5_hash == 'ed038e7ec59fa7b5504832c0b505d071'



"""Console script for RDP-Tools."""
import click
import os

from .collapse import collapse as collapse_reads
from .inflate import inflate_fasta, inflate_bam
from .client import RDPClient
from rich.pretty import pprint




def generate_collapse_filename(ctx, param, value):
    '''
    Generate the output filename for the collapse command

    Args:
        ctx (click.Context): Click context object
        param (click.Parameter): Click parameter object
        value (str): Output filename

    Returns:
        str: Output filename
    '''
    if value is not None:
        return value  # If --output is specified, use the provided filename
    if ctx.params['infile']:
        input_filename = ctx.params['infile']
        # Generate the default output filename by replacing the extension
        base_name, _ = os.path.splitext(input_filename)
        return f"{base_name}.collapsed.fa"
    return 'output.collapsed.fasta'  # If --output and input_file are not specified, use '.fasta'


def generate_inflate_filename(ctx, param, value):
    '''
    Generate the output filename for the inflate command

    Args:
        ctx (click.Context): Click context object
        param (click.Parameter): Click parameter object
        value (str): Output filename

    Returns:
        str: Output filename
    '''
    if value is not None:
        return value  # If --output is specified, use the provided filename
    if ctx.params['infile']:
        input_filename = ctx.params['infile']
        # Generate the default output filename by replacing the extension
        base_name, extension = os.path.splitext(input_filename)
        if extension == '.bam':
            return f"{base_name}.inflated.bam"
        else:
            return f"{base_name}.inflated.fa"
    return 'output.inflated.fasta'  # If --output and input_file are not specified, use '.fasta'


@click.group()
def rdp_tools():
    pass

@rdp_tools.command()
@click.argument('infile')
@click.option('--output', '-o', callback=generate_collapse_filename, help='Path to the output Fasta file')
@click.option('--format', '-f', default=f">seqREAD_xCOUNT", help='Custom header format')
@click.option('--compress', '-c', is_flag=True, help='Compress the output file')
def collapse(infile, output, format, compress):
    click.echo(f"Input file: {infile}")
    click.echo(f"Output file: {output}")
    click.echo("Collapsing...")
    collapse_reads(infile, output, format, compress)


@rdp_tools.command()
@click.argument('infile')
@click.option('--output', '-o', callback=generate_inflate_filename, help='Path to the output file')
@click.option('--format', '-f', default=f">seqREAD_xCOUNT", help='Custom header format')
@click.option('--compress', '-c', is_flag=True, help='Compress the output file')
def inflate(infile, output, format, compress):
    if output.endswith('bam'):
        if compress:
            click.echo("BAM files are already compressed. Ignoring --compress flag.")
        click.echo("Inflating...")
        inflate_bam(infile, output, format)
    else:
        click.echo("Inflating...")
        inflate_fasta(infile, output, format, compress)

@rdp_tools.group()
def database():
    pass

@database.command()
def sample_fields():
    # Call the method to list sample fields
    client = RDPClient()
    result = client.list_sample_fields()
    pprint(result, expand_all=True)



@database.command()
@click.option('--query', help='Specify the query parameters, e.g. "TISSUE=Lung"')
@click.option('--fields', help='List of DB fields to return, e.g. "TISSUE,CELL_LINE"')
@click.option('--limit', default=100, help='Limit the number of results')
def samples(query, fields, limit):
    # Call the method to query samples
    client = RDPClient()
    result = client.query_samples(query, fields, limit)
    pprint(result, expand_all=True)


if __name__ == '__main__':
    rdp_tools()



from Bio.SeqIO.FastaIO import SimpleFastaParser
import gzip
import pysam


def get_read_count(header: str, format: str = 'seqREAD_xCOUNT') -> int:
    '''
    Extract the read count from the read header

    Args:
        header (str): Read header
        format (str, optional): The layout of the read file headers. Defaults to 'seqREAD_xCOUNT'.

    Raises:
        ValueError: If the header format is not recognized.

    Returns:
        int: The read count
    '''
    delimiter = format.split('READ')[1].split('COUNT')[0]
    read_count = header.split(delimiter)[1].split('COUNT')[0]
    return int(read_count)


def write_read(header: str, count: int, sequence: str, out: str, out_format: str) -> None:
    """
    Write a read to the output file.

    Args:
        header (str): The read header.
        count (int): The read count.
        sequence (str): The read sequence.
        out (str): The path to the output file.
        out_format (str): The output format.

    Returns:
        None
    """
    for i in range(count):
        if out_format == 'fasta':
            out.write(f'>{header}\n{sequence}\n')
        elif out_format == 'fastq':
            out.write(f'@{header}\n{sequence}\n+\n{"I" * len(sequence)}\n')


def inflate_fasta(infile: str, outfile: str, format: str = 'seqREAD_xCOUNT', compress: bool = False) -> None:
    """
    Inflate the contents of a FASTA file based on the read count in the read name.

    Args:
        infile (str): The path to the input FASTA file.
        outfile (str): The path to the output file. Extension determines the output format. 
        format (str, optional): The layout of the read file headers. Defaults to 'seqREAD_xCOUNT'.
        compress (bool, optional): Compress the output file. Defaults to False.
    Raises:
        ValueError: If the output format is not 'fasta' or 'fastq'.

    Returns:
        None
    """
    if any(outfile.strip('.gz').endswith(ext) for ext in ['fa', 'fasta', 'fna']):
        out_format = 'fasta'
    elif any(outfile.strip('.gz').endswith(ext) for ext in ['fq', 'fastq']):
        out_format = 'fastq'
    else:
        raise ValueError("Output format must be 'fasta' ['fa', 'fasta', 'fna'] or 'fastq' ['fq', 'fastq']")
    
    try:
        f = gzip.open(infile, 'rt')
    except OSError:
        f = open(infile, 'r')
        
    # write over existing file
    open(outfile, 'w').close()

    inflated_count = 1
    for header, sequence in SimpleFastaParser(f):
        read_count = get_read_count(header, format)

        if compress:
            with gzip.open(outfile, 'at') as out:
                write_read(header, read_count, sequence, out, out_format)
        else:
            with open(outfile, 'a') as out:
                write_read(header, read_count, sequence, out, out_format)
                inflated_count += 1


def inflate_bam(infile: str, outfile: str, format: str = 'seqREAD_xCOUNT') -> None:
    """
    Inflate the contents of a BAM file based on the read count in the read name.

    Args:
        infile (str): The path to the input BAM file.
        outfile (str): The path to the output file. 
        format (str, optional): The layout of the read headers. Defaults to 'seqREAD_xCOUNT'.

    Returns:
        None
    """
    with pysam.AlignmentFile(infile, 'rb') as bam:
        with pysam.AlignmentFile(outfile, 'wb', template=bam) as out:
            for read in bam:
                read_count = get_read_count(read.qname, format)
                for i in range(read_count):
                    out.write(read)